#include <IRremote.h>

#define MIN(a,b) (((a)<(b))?(a):(b))
#define MAX(a,b) (((a)>(b))?(a):(b))

#define MAX_LIGHT_VALUE 1017.0
#define MIN_LIGHT_VALUE 344.0
#define DARK_THRESHOLD  4.8

#define VOLUME_UP     0xE0E0E01F
#define VOLUME_DOWN   0xE0E0D02F
#define ON            0xE0E020DF
#define OFF           0xE0E0A05F
#define NORMAL        0xE0E08877

#define ON_VALUE      1
#define OFF_VALUE     2
#define NORMAL_VALUE  0

#define PIR_PIN       13
#define LIGHT_OUT_PIN 6
#define IR_PIN        A4
#define LIGHT_IN_PIN  A5

IRrecv irrecv(IR_PIN);
decode_results results;

int dutyCycle = 127; // 50% initial duty-cycle
int lightMode; // Normal, Always on, Always off

bool motion = false;
bool shouldLight = false;

void checkMotion() {
    bool newMotion = digitalRead(PIR_PIN) == HIGH;

    if (!motion && newMotion) {
        shouldLight = true;
    } else if (motion && !newMotion) {
        TCNT1 = 0;
    } else if (motion && newMotion) {
        TCNT1 = 0;
    }

    motion = newMotion;
}

bool isDark() {
  float convertedValue = 
      (analogRead(LIGHT_IN_PIN) - MIN_LIGHT_VALUE) 
        / (MAX_LIGHT_VALUE - MIN_LIGHT_VALUE) * 5.0;

    return convertedValue >= DARK_THRESHOLD;
}

ISR(TIMER1_COMPA_vect) {
    shouldLight = false;
    Serial.println("Timer: Light off");
}

void setup() {
    // Pin setup
    pinMode(PIR_PIN, INPUT);
    pinMode(LIGHT_OUT_PIN, OUTPUT);

    // PWM Timer 0
    TCCR0A = 0; TCCR0B = 0;
    TCCR0A |= (1 << WGM01) | (1 << WGM00); // fast PWM
    TCCR0A |= (1 << COM0A1);               // non inverting mode
    TCCR0B |= (1 << CS02) | (1 << CS00);   // 1024 prescaler
    OCR0A = dutyCycle;                     // duty-cycle

    irrecv.enableIRIn();            // Start the receiver

    // Timer 1
    cli();
  
    TCCR1A = 0; TCCR1B = 0; TCNT1 = 0;
    OCR1A = 62500;                          // compare match register 16MHz/1024/0.25Hz
    TCCR1B |= (1 << WGM12);                 // CTC mode
    TCCR1B |= (1 << CS12) | (1 << CS10);    // 1024 prescaler 
    TIMSK1 |= (1 << OCIE1A);                // enable timer compare interrupt
    sei();
    
    Serial.begin(9600);
    Serial.println("MODE: Normal");
}

void loop() {
    // Update PIR flag
    checkMotion();

    // Check for IR commands
    if (irrecv.decode(&results)) {
        Serial.print("Key code detected: ");
        Serial.println(results.value, HEX);
        
        if (results.value == VOLUME_UP)
            dutyCycle = MIN(255, dutyCycle + 25);
        if (results.value == VOLUME_DOWN)
            dutyCycle = MAX(0, dutyCycle - 25);
        if (results.value == ON) {
            lightMode = ON_VALUE;
            Serial.println("MODE: ON");
        }
        if (results.value == OFF) {
            lightMode = OFF_VALUE;
            Serial.println("MODE: OFF");
        }
        if (results.value == NORMAL) {
            lightMode = NORMAL_VALUE;
            Serial.println("MODE: Normal");
        }
          
        irrecv.resume(); // Receive the next value
    }

    // Set the duty-cycle according to the operating mode
    switch (lightMode) {
    case NORMAL_VALUE:
        OCR0A = isDark() && shouldLight ? dutyCycle : 0;
        break;
    case ON_VALUE:
        OCR0A = dutyCycle;
        break;
    case OFF_VALUE:
        OCR0A = 0;
        break;
    }   
}
